//We run a continuous speedcheck to do a bunch of stuff like speed-related sound and driver direction
function Mat64vehicleSpeedCheck(%this, %obj)
{
    if(!isObject(%obj))
        return;
 
    %vehicle = %obj;
 
    %slot = 0; //Play the sound in the driver's spot
    %vehicle.speed = vectorDot(%vehicle.getVelocity(), %vehicle.getForwardVector());
    commandtoclient(%vehicle.getControllingClient(),'bottomprint', mAbs(mFloor(%vehicle.speed * 2)) @"<font:impact:25><color:FFFFFF> BPS", 1);
    if(!%vehicle.oldSpeed)
        %vehicle.oldSpeed = %vehicle.speed;
 
    //Braking is not working currently
    if(%vehicle.oldSpeed)
    {
        if(mAbs(%vehicle.oldSpeed) - mAbs(%vehicle.speed) > 1) //Braking rate of speed in 500MS
            %applyBrakes = true;
 
        if(mAbs(%vehicle.oldSpeed) - mAbs(%vehicle.speed) < 1) //We take absolute values here to prevent the train from always 'braking' when in reverse
            %applyBrakes = false;
    }
 
    %vehicle.oldSpeed = %vehicle.speed;
 
    //We use else if's so that it will find a true statement then skip the rest
    if(%vehicle.speed < 2)
        %vehicle.playAudio(%slot, throttle0Sound);		//play idle sound when the train is stopped
 
    else if(%vehicle.speed < 10)
        %vehicle.playAudio(%slot, throttle1Sound);		//play progressively higher pitched sounds if speed increases
 
    else if(%vehicle.speed < 20)
        %vehicle.playAudio(%slot, throttle2Sound);
 
    else if(%vehicle.speed < 30)
        %vehicle.playAudio(%slot, throttle3Sound);
 
    else if(%vehicle.speed < 40)
        %vehicle.playAudio(%slot, throttle4Sound);
 
    else if(%vehicle.speed < 50)
        %vehicle.playAudio(%slot, throttle5Sound);
 
    else if(%vehicle.speed < 70)
        %vehicle.playAudio(%slot, throttle6Sound);
 
	if(%vehicle.speed >=-1)			//Is the train going forward?
	{
		%vehicle.playThread(3, rotfront);		//Play a 1-frame animation rotating mount0 (driver) and mount15 (guy standing next to driver) to the front cab
		%obj.hideNode(HL_Wht_0);				//Hide and show the proper 'light' nodes on the front side of the train
		%obj.unhideNode(HL_Wht_1);
		%obj.unhideNode(HL_Red_0);
		%obj.hideNode(HL_Red_1);
		
		%obj.hideNode(TL_Wht_1);				//Same on the back of the train
		%obj.unhideNode(TL_Wht_0);
		%obj.unhideNode(TL_Red_1);
		%obj.hideNode(TL_Red_0);
	}
	else
	{
		%vehicle.playThread(3, rotback);		//If the train is going backwards, play 1-frame animation rotating mount0 and mount15 180 degrees to the other side of the train
		%obj.unhideNode(HL_Wht_0);				//Again show the proper lights
		%obj.hideNode(HL_Wht_1);
		%obj.hideNode(HL_Red_0);
		%obj.unhideNode(HL_Red_1);
	
		%obj.unhideNode(TL_Wht_1);
		%obj.hideNode(TL_Wht_0);
		%obj.hideNode(TL_Red_1);
		%obj.unhideNode(TL_Red_0);
	}
 
    if(%applyBrakes)		//Play the squeal sound if braking
        %vehicle.playAudio(1, squealSound);
    if(!%applyBrakes)		//Stop playing that sound if we're no longer braking
        %vehicle.stopAudio(1);
       
    //Force the train back on its initial straight track if it deviates from it in position or rotation   
    %fullPos = %vehicle.getPosition();
   
    %actualPos = getWord(%vehicle.getPosition(), %vehicle.axisOfCorrection);
    %dif = mAbs(%actualPos - %vehicle.AutoCorrectPos);
   
    if(%dif > 0.02)
    {
        // Make Correction
        if(%vehicle.axisOfCorrection == 0)
        {
            // Correcting the X axis
           
            %x = %vehicle.AutoCorrectPos;
            %y = getWord(%fullPos, 1);
            %z = getWord(%fullPos, 2);
           
            %vehicle.setTransform(%x SPC %y SPC %z SPC %obj.AutoCorrectRot);
        }
       
        if(%vehicle.axisOfCorrection == 1)
        {
            // Correcting the Y axis
           
            %x = getWord(%fullPos, 0);
            %y = %vehicle.AutoCorrectPos;
            %z = getWord(%fullPos, 2);
           
            %vehicle.setTransform(%x SPC %y SPC %z SPC %obj.AutoCorrectRot);
        }
    }
	schedule(500,0,"Mat64vehicleSpeedCheck",%this,%obj);
}

function Mat64Vehicle::doorControl(%this, %obj, %direction)
{
	if (!isObject(%obj)) return;

	%obj.signal = %direction;	//Other functions must know which direction is being used

	if(%direction == 0)	//0 is a signal to close door
	{	//Close door
		return;
	}	//End of close door
		if(%direction == 1)		//Right door toggle
		{
			if(%obj.Rdoor $= "closed")
			{
				%obj.playThread(2, rdooropen);
				%obj.Rdoor = "open";
			}
			else
			{
				%obj.playThread(2, rdoorclose);
				%obj.Rdoor = "closed";
			}
		}
		else					//Left door toggle
		{
			
			if(%obj.Ldoor $= "closed")
			{
				%obj.playThread(1, ldooropen);
				%obj.Ldoor = "open";
			}
			else
			{
				%obj.playThread(1, ldoorclose);
				%obj.Ldoor = "closed";
			}
		}
}
 
function Mat64vehicle::onAdd(%this, %obj)
{
    parent::onadd(%this,%obj);
	
    Mat64vehicleSpeedCheck(%this, %obj);

	%obj.hideNode(HL_Wht_0);
	%obj.unhideNode(HL_Wht_1);
	%obj.unhideNode(HL_Red_0);
	%obj.hideNode(HL_Red_1);
	
	%obj.hideNode(TL_Wht_1);
	%obj.unhideNode(TL_Wht_0);
	%obj.unhideNode(TL_Red_1);
	%obj.hideNode(TL_Red_0);
	
	%obj.Ldoor = "closed";
	%obj.Rdoor = "closed";
	
	schedule(100,0,"TrainAutoCorrect_GetData",%this,%obj);
}
 
function axisToEuler(%axis)
{
    %angleOver2 = getWord(%axis,3) * 0.5;
    %angleOver2 = -%angleOver2;
    %sinThetaOver2 = mSin(%angleOver2);
    %cosThetaOver2 = mCos(%angleOver2);
    %q0 = %cosThetaOver2;
    %q1 = getWord(%axis,0) * %sinThetaOver2;
    %q2 = getWord(%axis,1) * %sinThetaOver2;
    %q3 = getWord(%axis,2) * %sinThetaOver2;
    %q0q0 = %q0 * %q0;
    %q1q2 = %q1 * %q2;
    %q0q3 = %q0 * %q3;
    %q1q3 = %q1 * %q3;
    %q0q2 = %q0 * %q2;
    %q2q2 = %q2 * %q2;
    %q2q3 = %q2 * %q3;
    %q0q1 = %q0 * %q1;
    %q3q3 = %q3 * %q3;
    %m13 = 2.0 * (%q1q3 - %q0q2);
    %m21 = 2.0 * (%q1q2 - %q0q3);
    %m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
    %m23 = 2.0 * (%q2q3 + %q0q1);
    %m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
    return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

function Mat64Vehicle::onRemove(%data, %obj)
{
	cancel(%obj.Mat64vehicleSpeedCheck);
	cancel(%obj.TrainAutoCorrect_GetData);
}
 
 
//Get the intended line and angle the train is meant to be at
function TrainAutoCorrect_GetData(%this, %obj)
{
    %spawnBrick             = %obj.spawnBrick;
    %spawnBrickTransform    = %spawnBrick.getTransform();
    %spawnBrickRotation     = getWords(%spawnBrickTransform, 3, 6);
    %spawnBrickEuler        = axisToEuler(%spawnBrickRotation);
    %zRot                   = mFloor(mAbs(getWord(%spawnBrickEuler, 2)));
	
    if(%zRot == 90)
        %obj.axisOfCorrection = 1;
    else
        %obj.axisOfCorrection = 0;
	
    %obj.AutoCorrectRot = %spawnBrickRotation;
    %obj.AutoCorrectPos = getWord(%spawnBrickTransform, %obj.axisOfCorrection);
}

function Mat64vehicle::onCollision(%this, %obj, %col, %vec, %vecLen)
	{
		%speed2 = vectorLen(%obj.getVelocity());
		if(isObject(%Col.Client.player) && %speed2 > 10)
		{
			%Col.kill();
		}
		if(%Col.getclassname() $= "AiPlayer" && %speed2 > 10)
		{
			%Col.kill();
		}
		return parent::onCollision(%this, %obj, %col, %vec, %vecLen);
	}